<template>
  <Popup
    ref="pop"
    :title="title"
    :left="left"
    width="390px"
    showBtn="true"
    :btnArr="btnArr"
    @yes="yes"
    @cancel="close"
  >
    <el-form ref="form" :model="data" :rules="rules" label-width="100px">
      <div v-if="isLogin">
        <el-form-item label="地址:" prop="url">
          <el-input v-model="data.url"></el-input>
        </el-form-item>
        <el-form-item label="用户名:" prop="username">
          <el-input v-model="data.username"></el-input>
        </el-form-item>
        <el-form-item label="密码:" prop="password">
          <el-input v-model="data.password" show-password></el-input>
        </el-form-item>
      </div>
      <div v-else>
        <el-form-item label="服务类型:">
          <el-select v-model="type">
            <el-option label="WMS" value="WMS"></el-option>
            <el-option label="WFS" value="WFS"></el-option>
            <el-option label="TMS" value="TMS"></el-option>
            <el-option label="C3D" value="C3D"></el-option>
            <el-option label="CTS" value="CTS"></el-option>
          </el-select>
        </el-form-item>
        <div
          class="mapList scrollbar"
          v-loading="loading"
          element-loading-background="rgba(0, 0, 0, 0.5)"
        >
          <el-tree ref="tree" :data="layers[type]"></el-tree>
        </div>
      </div>
    </el-form>
  </Popup>
</template>

<script>
import Popup from "@tools/Popup";
import serviceTools from "@mixin/serviceTools";
import axios from "axios";

// axios.defaults.withCredentials = true;
export default {
  name: "Gisserver",
  components: {
    Popup,
  },
  mixins: [serviceTools],
  data() {
    return {
      title: "Gisserver",
      left: undefined,
      key: "",
      layers: {
        WMS: [],
        WFS: [],
        TMS: [],
        C3D: [],
        CTS: [],
      },
      isLogin: true,
      loading: false,
      type: "WMS",
      data: {
        url: "http://183.162.245.49:3301/gisserver",
        username: "",
        password: "",
      },
      // 表单验证规则
      rules: {
        username: [
          { required: true, message: "请输入用户名", trigger: "blur" },
        ],
        password: [{ required: true, message: "请输入密码", trigger: "blur" }],
        url: [{ required: true, message: "请输入服务地址", trigger: "blur" }],
      },
      btnArr: [],
    };
  },
  computed: {},
  mounted() {
    if (sessionStorage.getItem("gisserverURL")) {
      this.data.url = sessionStorage.getItem("gisserverURL");
    }
    if (
      sessionStorage.getItem("gisserverURL") &&
      sessionStorage.getItem("gisserverKey")
    ) {
      this.key = sessionStorage.getItem("gisserverKey");
      this.getLayers();
      this.isLogin = false;
      this.setBtnArr();
    }
  },
  methods: {
    setBtnArr() {
      if (this.isLogin) {
        this.btnArr = [];
      } else {
        this.btnArr = [
          {
            name: "退出",
            fun: () => {
              this.isLogin = true;
              sessionStorage.removeItem("gisserverKey");
              this.key = "";
              this.btnArr = [];
            },
          },
          {
            name: "取消",
            fun: () => {
              this.close();
            },
          },
          {
            name: "确定",
            type: "primary",
            fun: () => {
              this.yes();
            },
          },
        ];
      }
    },
    // 关闭弹窗
    close() {
      this.$refs.pop.close();
    },
    // 打开弹窗
    open() {
      this.$refs.pop.open();
    },
    // 添加数据
    yes() {
      this.$refs.form.validate((valid) => {
        // 验证通过
        if (valid) {
          if (this.isLogin) {
            this.login();
          } else {
            this.addLayer();
          }
        }
      });
    },
    addLayer() {
      let selectNode = this.$refs.tree && this.$refs.tree.getCurrentNode();
      if (selectNode) {
        if (selectNode.status !== "running") {
          this.$message("服务未启动！");
        } else {
          let data = {
            id: SmartEarth.uuid(),
            name: selectNode.title,
            urls: this.data.url,
          };
          switch (this.type) {
            case "WMS":
              data = {
                ...data,
                sourceType: "wms",
                layer: "",
              };
              data.urls += `/wmsserver/${selectNode.name}`;
              break;
            case "C3D":
              data = {
                ...data,
                sourceType: "b3dm",
              };
              data.urls += `/c3dserver/${selectNode.name}/tileset.json`;
              break;
            case "CTS":
              data = {
                ...data,
                sourceType: "terrain",
              };
              data.urls += `/ctsserver/${selectNode.name}`;
              break;
            case "TMS":
              data = {
                ...data,
                sourceType: "local-map",
                tms: true,
                openProp: true,
              };
              data.urls += `/tmsserver/${selectNode.name}`;
              break;
            case "WFS":
              data = {
                ...data,
                sourceType: "wfs",
                openProp: true,
              };
              data.urls += `/wfsserver/${selectNode.name}`;
              break;
          }
          if (this.type === "WMS" || this.type === "CTS") {
            axios
              .get(
                this.data.url + `/rest/services/${selectNode.name}/MapServer`
              )
              .then((ref) => {
                let fullExtent = ref && ref.data && ref.data.fullExtent;
                if (fullExtent) {
                  if (
                    fullExtent.spatialReference &&
                    fullExtent.spatialReference.wkid !== 4326 &&
                    fullExtent.spatialReference.wkid !== 4490
                  ) {
                    this.$message(
                      `暂不支持EPSG:${fullExtent.spatialReference.wkid}坐标系`
                    );
                    return;
                  }
                  data.flyTo = [
                    fullExtent.xmin,
                    fullExtent.ymin,
                    fullExtent.xmax,
                    fullExtent.ymax,
                  ];
                }
                this.$emit("success", data);
                this.close();
              })
              .catch(() => {
                this.$emit("success", data);
                this.close();
              });
          } else {
            this.$emit("success", data);
            this.close();
          }
        }
      } else {
        this.$message("请选择需要加载的图层");
      }
    },
    login() {
      axios
        .post(this.data.url + "/login", null, {
          params: {
            username: this.data.username,
            password: this.data.password,
            plain: true,
            f: "json",
          },
        })
        .then((data) => {
          if (data.data && data.data.sessionid) {
            this.$message({
              message: "登录成功",
              type: "success",
            });
            this.key = data.data.sessionid;
            sessionStorage.setItem("gisserverURL", this.data.url);
            sessionStorage.setItem("gisserverKey", this.key);
            this.getLayers();
            this.isLogin = false;
            this.setBtnArr();
          } else {
            this.$message({
              message: "用户名或密码错误",
              type: "error",
            });
          }
        })
        .catch(() => {
          this.$message({
            message: "请输入正确的服务地址",
            type: "warning",
          });
        });
    },
    getLayers() {
      this.loading = true;
      this.layers = {
        WMS: [],
        WFS: [],
        TMS: [],
        C3D: [],
        CTS: [],
      };
      this.getGisserverLayerList(this.data.url.replace("/gisserver", "")).then(
        (data) => {
          this.layers = data.layers;
          this.loading = false;
        }
      );
    },
  },
};
</script>

<style scoped lang="less">
.el-form {
  margin-top: 20px;
  margin-right: 10px;
  width: 380px;

  /deep/ .el-form-item__label {
    color: #fff;
    font-size: 18px;
  }

  /deep/ .el-select {
    width: 100%;
  }

  .mapList {
    border-top: 1px solid rgba(32, 160, 255, 0.3);
    max-height: 200px;
    padding-top: 5px;
  }

  .el-tree {
    background: transparent;
    color: #fff;
    margin: 0 10px;
  }

  /deep/ .el-tree-node__content:hover {
    background-color: rgba(245, 247, 250, 0.2);
  }

  /deep/ .el-tree-node:focus > .el-tree-node__content {
    background-color: rgba(245, 247, 250, 0.2);
  }
}
</style>
